@@ -19,8 +19,8 @@ from tkinter import Tk
 from ._cluster_merger_gui import ClusterMergerGUI
 
 
-def sig_fig_rounder(data: list, unc: list):
-    """Rounds the data based on its uncertainty according to
+def sig_fig_rounder(data: list, err: list):
+    """Rounds the data based on its standard error according to
     standard significant figure rules.
 
     Parameters
@@ -28,70 +28,71 @@ def sig_fig_rounder(data: list, unc: list):
     data : list, len(n_samples)
         The data to be rounded.
 
-    unc : list, len(n_samples)
-        The uncertainty in the data.
+    err : list, len(n_samples)
+        The standard error in the data.
     """
-    for u in range(len(unc)):
-        if unc[u] <= 0:
-            unc[u] = 0.001
-        unc_string = str(unc[u])
-        unc_string = unc_string.replace('0', '')
-        unc_string = unc_string.replace('.', '')
-        data_string = str(data[u])
+    for n in range(len(err)):
+        if err[n] <= 0:
+            err[n] = 0.001  # Ensure all errors are positive
+        err_string = str(err[n])
+        err_string = err_string.replace('0', '')
+        err_string = err_string.replace('.', '')
+        data_string = str(data[n])
         data_string = data_string.replace('0', '')
         data_string = data_string.replace('.', '')
         if data_string[-1] == '5':
-            data_string = str(data[u]) + '1'
-            data[u] = float(data_string)
-        if unc_string[0] != '1':
-            if unc_string[-1] == '5':
-                unc_string = str(unc[u]) + '1'
-                unc[u] = float(unc_string)
-            decimal_places = -int(np.floor(np.log10(unc[u])))
-            data[u] = str(round(data[u], decimal_places))
-            unc[u] = str(round(unc[u], decimal_places))
-            if len(data[u]) < abs(decimal_places) + 2 and \
-                    data[u][0] != '-':
-                while len(data[u]) != abs(decimal_places) + 2:
-                    data[u] += '0'
-            if len(data[u]) < abs(decimal_places) + 3 and \
-                    data[u][0] == '-':
-                while len(data[u]) != abs(decimal_places) + 3:
-                    data[u] += '0'
-            if len(unc[u]) < abs(decimal_places) + 2 and \
-                    unc[u][-1] != '2':
-                while len(unc[u]) != abs(decimal_places) + 2:
-                    unc[u] += '0'
-            if float(unc[u]) >= 2:
-                data[u] = str(int(round(float(data[u]), 0)))
-                unc[u] = str(int(round(float(unc[u]), 0)))
+            data_string = str(data[n]) + '1'
+            data[n] = float(data_string)
+        if err_string[0] != '1':
+            if err_string[-1] == '5':
+                err_string = str(err[n]) + '1'
+                err[n] = float(err_string)
+            decimal_places = -int(np.floor(np.log10(err[n])))
+            data[n] = str(round(data[n], decimal_places))
+            err[n] = str(round(err[n], decimal_places))
+            if len(data[n]) < abs(decimal_places) + 2 and \
+                    data[n][0] != '-':
+                while len(data[n]) != abs(decimal_places) + 2:
+                    data[n] += '0'
+            if len(data[n]) < abs(decimal_places) + 3 and \
+                    data[n][0] == '-':
+                while len(data[n]) != abs(decimal_places) + 3:
+                    data[n] += '0'
+            if len(err[n]) < abs(decimal_places) + 2 and \
+                    err[n][-1] != '2':
+                while len(err[n]) != abs(decimal_places) + 2:
+                    err[n] += '0'
+            if float(err[n]) >= 2:
+                data[n] = str(int(round(float(data[n]), 0)))
+                err[n] = str(int(round(float(err[n]), 0)))
         else:
-            if unc_string[-1] == '5':
-                unc_string = str(unc[u]) + '1'
-                unc[u] = float(unc_string)
-            decimal_places = 1 - int(np.floor(np.log10(unc[u])))
-            data[u] = str(round(data[u], decimal_places))
-            unc[u] = str(round(unc[u], decimal_places))
-            test_unc = unc[u].replace('0', '')
-            test_unc = test_unc.replace('.', '')
-            if test_unc[0] == '2':
+            if err_string[-1] == '5':
+                err_string = str(err[n]) + '1'
+                err[n] = float(err_string)
+            decimal_places = 1 - int(np.floor(np.log10(err[n])))
+            data[n] = str(round(data[n], decimal_places))
+            err[n] = str(round(err[n], decimal_places))
+            test_err = err[n].replace('0', '')
+            test_err = test_err.replace('.', '')
+            if test_err[0] == '2':
                 decimal_places -= 1
-            if len(data[u]) < abs(decimal_places) + 2 and \
-                    data[u][0] != '-':
-                while len(data[u]) != abs(decimal_places) + 2:
-                    data[u] += '0'
-            if len(data[u]) < abs(decimal_places) + 3 and \
-                    data[u][0] == '-':
-                while len(data[u]) != abs(decimal_places) + 3:
-                    data[u] += '0'
-            if len(unc[u]) < abs(decimal_places) + 2 and \
-                    unc[u][-1] != '2':
-                while len(unc[u]) != abs(decimal_places) + 2:
-                    unc[u] += '0'
-            if float(unc[u]) >= 2:
-                data[u] = str(int(round(float(data[u]), 0)))
-                unc[u] = str(int(round(float(unc[u]), 0)))
-    return data, unc
+            if len(data[n]) < abs(decimal_places) + 2 and \
+                    data[n][0] != '-':
+                while len(data[n]) != abs(decimal_places) + 2:
+                    data[n] += '0'
+            if len(data[n]) < abs(decimal_places) + 3 and \
+                    data[n][0] == '-':
+                while len(data[n]) != abs(decimal_places) + 3:
+                    data[n] += '0'
+            if len(err[n]) < abs(decimal_places) + 2 and \
+                    err[n][-1] != '2':
+                while len(err[n]) != abs(decimal_places) + 2:
+                    err[n] += '0'
+            if float(err[n]) >= 2:
+                data[n] = str(int(round(float(data[n]), 0)))
+                err[n] = str(int(round(float(err[n]), 0)))
+
+    return data, err
 
 
 class GaussianMixtureBase(metaclass=ABCMeta):
@@ -103,7 +104,6 @@ class GaussianMixtureBase(metaclass=ABCMeta):
 
     version : 0.1
     """
-
     def __init__(self, n_components, cov_type, tol, max_iter,
                  n_init):
         self.n_components = n_components
@@ -155,38 +155,76 @@ class GaussianMixtureBase(metaclass=ABCMeta):
         pass
 
     @abstractmethod
-    def fit_over_one_dimensional_histograms(self, fig, axs,
-                                            data_frame_object):
-        """Fit over the histograms generated with the data frame object.
+    def _calc_secondary_centers_unc(self, c1s, c1s_err, c2s,
+                                    c2s_err, data_frame_object):
+        """Calculate the coordinates of the cluster centers for the coordinate system
+        that was not used for the fit.
 
-        Given a data frame object that has already been used
-        to generate histograms for each dimension of data, this
-        method will graph a GMM fit over each dimension. The returned
-        matplotlib.plyplot figure can be shown and saved separately.
+        Standard errors are calculated with typical standard error propagation methods.
 
         Parameters
         ----------
-        fig : matplotlib.pyplot figure
-            The overarching figure object.
+        c1s : array-like, shape (n_components,)
+            The x-coordinates of the cluster centers if clustering
+            with Cartesian coordinates, otherwise the r-coordinates
+            of the cluster centers.
 
-        axs : matplotlib.pyplot axes
-            Contains the four different histograms.
+        c1s_err : array-like, shape (n_components,)
+            The standard error in the c1s.
 
-        data_frame_object : object from the class DataFrame
+        c2s : array-like, shape (n_components,)
+            The y-coordinates of the cluster centers if clustering
+            with Cartesian coordinates, otherwise the p-coordinates
+            of the cluster centers.
+
+        c2s_err : array-like, shape (n_components,)
+            The standard error in the c2s.
+
+        data_frame_object : DataFrame class object
             The object that contains the processed data and
             information that is used by the algorithms to do the
             fits.
-
-        Returns
-        -------
-        fig : matplotlib.pyplot figure
-            The overarching figure object.
         """
         pass
 
     @abstractmethod
-    def cluster_data(self, data_frame_object):
-        """Use the Gaussian Mixture Model fit from the sklearn package to cluster the data.
+    def _calculate_centers_uncertainties(self, data_frame_object):
+        """After clustering the data, organize the cluster centers into a more accessible format.
+
+        Assigns the attributes 'centers_array_', 'ips_',
+        'unique_labels', and 'colors_'.
+
+        Parameters
+        ----------
+        data_frame_object : DataFrame class object
+            The object that contains the processed data and
+            information that is used by the algorithms to do the
+            fits.
+        """
+        pass
+
+    @abstractmethod
+    def recalculate_centers_uncertainties(self, data_frame_object):
+        """Recalculate the centers of each cluster and the uncertainties in the centers.
+
+        This uses a different method from simply extracting the centers
+        and uncertainties from the fit. Instead, it fits a univariate Gaussian
+        to each dimension of each cluster and uses the statistics from the
+        fits to calculate the centers. This method was written by Dwaipayan
+        Ray and Adrian Valverde.
+
+        Parameters
+        ----------
+        data_frame_object : DataFrame class object
+            The object that contains the processed data and
+            information that is used by the algorithms to do the
+            fits.
+        """
+        pass
+
+    @abstractmethod
+    def _cluster_data_one_d(self, x):
+        """Use the Gaussian Mixture fits from the sklearn package to cluster the data.
 
         Assigns the object the attributes 'means_',
         'covariances_', 'weights_', 'labels_', 'responsibilities_',
@@ -194,7 +232,22 @@ class GaussianMixtureBase(metaclass=ABCMeta):
 
         Parameters
         ----------
-        data_frame_object : object from the class DataFrame
+        x : array-like, shape (n_samples, n_attributes)
+            The data to be clustered.
+        """
+        pass
+
+    @abstractmethod
+    def cluster_data(self, data_frame_object):
+        """Use the Gaussian Mixture fits from the sklearn package to cluster the data.
+
+        Assigns the object the attributes 'means_',
+        'covariances_', 'weights_', 'labels_', 'responsibilities_',
+        and 'n_comps_found_'.
+
+        Parameters
+        ----------
+        data_frame_object : DataFrame class object
             The object that contains the processed data and
             information that is used by the algorithms to do the
             fits.
@@ -211,38 +264,7 @@ class GaussianMixtureBase(metaclass=ABCMeta):
 
         Parameters
         ----------
-        data_frame_object : object from the class DataFrame
-                The object that contains the processed data and
-                information that is used by the algorithms to do the
-                fits.
-        """
-        pass
-
-    @abstractmethod
-    def _cluster_data_one_d(self, x):
-        """Use the Gaussian Mixture Model fit from the sklearn package to cluster the data.
-
-        Assigns the object the attributes 'means_',
-        'covariances_', 'weights_', 'labels_', 'responsibilities_',
-        and 'n_comps_found_'.
-
-        Parameters
-        ----------
-        x : array-like, shape (n_samples, n_attributes)
-            The data to be clustered
-        """
-        pass
-
-    @abstractmethod
-    def _calculate_centers_uncertainties(self, data_frame_object):
-        """After clustering the data, organize the cluster centers into a more accessible format.
-
-        Assigns the attributes 'centers_array_', 'ips_',
-        'unique_labels', and 'colors_'.
-
-        Parameters
-        ----------
-        data_frame_object : object from the class DataFrame
+        data_frame_object : DataFrame class object
             The object that contains the processed data and
             information that is used by the algorithms to do the
             fits.
@@ -250,22 +272,33 @@ class GaussianMixtureBase(metaclass=ABCMeta):
         pass
 
     @abstractmethod
-    def recalculate_centers_uncertainties(self, data_frame_object):
-        """Recalculate the centers of each cluster and the uncertainties in the centers.
+    def fit_over_one_dimensional_histograms(self, fig, axs,
+                                            data_frame_object):
+        """Fit over the histograms generated with the data frame object.
 
-        This uses a different method from simply extracting the centers
-        and uncertainties from the fit. Instead, it fits a univariate Gaussian
-        curve to each dimension of each cluster and uses the
-        statistics from the fits to calculate the centers.
-
-        This method was written by Dwaipayan Ray and Adrian Valverde.
+        Given a data frame object that has already been used
+        to generate histograms for each dimension of data, this
+        method will graph a GMM fit over each dimension. The returned
+        matplotlib.plyplot figure may be shown with the method plt.show()
+        or saved with the method plt.savefig() separately.
 
         Parameters
         ----------
-        data_frame_object : object from the class DataFrame
+        fig : matplotlib.pyplot figure
+            The overarching figure object.
+
+        axs : matplotlib.pyplot axes
+            Contains the four different histograms.
+
+        data_frame_object : DataFrame class object
             The object that contains the processed data and
             information that is used by the algorithms to do the
             fits.
+
+        Returns
+        -------
+        fig : matplotlib.pyplot figure
+            The overarching figure object.
         """
         pass
 
@@ -278,7 +311,7 @@ class GaussianMixtureBase(metaclass=ABCMeta):
 
         Parameters
         ----------
-        data_frame_object : object from the class DataFrame
+        data_frame_object : DataFrame class object
             The object that contains the processed data and
             information that is used by the algorithms to do the
             fits.
@@ -298,10 +331,10 @@ class GaussianMixtureBase(metaclass=ABCMeta):
     def show_results(self, data_frame_object):
         """Return the clustering results.
 
-        The returned matplotlib.plyplot figure can be shown and saved
-        separately.
-
-        This method was written by Dwaipayan Ray and Adrian Valverde.
+        The returned matplotlib.plyplot figure may be shown
+        with the method plt.show() or saved with the method
+        plt.savefig() separately. This method was written by
+        Dwaipayan Ray and Adrian Valverde.
 
         Parameters
         ----------
@@ -389,13 +422,13 @@ class GaussianMixtureBase(metaclass=ABCMeta):
                           'lightgray']
 
                 # Find indexes of colors
-                true_index_list = []
-                cluster_index_list = []
+                true_index_list = []  # Corresponds to the indices of the clusters in centers_array
+                cluster_index_list = []  # Corresponds to the indices of the clusters relative to labels_ and colors
                 for color in color_list:
                     true_index_list.append(self.colors_.index(color))
                     cluster_index_list.append(colors.index(color))
-                clusters_lost = len(true_index_list) - 1
-                true_index_keep = self.ips_.tolist().index(max(self.ips_[true_index_list]))
+                ips_list = list(self.ips_)
+                true_index_keep = ips_list.index(max(self.ips_[true_index_list]))
                 cluster_index_keep = cluster_index_list[true_index_list.index(true_index_keep)]
                 other_true_indices = [i for i in true_index_list if i != true_index_keep]
                 other_true_indices.sort(reverse=True)
@@ -409,7 +442,7 @@ class GaussianMixtureBase(metaclass=ABCMeta):
                 for i in other_cluster_indices:
                     self.labels_ = np.where(self.labels_ == i, cluster_index_keep, self.labels_)
                 self.unique_labels_ = np.unique(self.labels_)
-                labels_list = self.labels_.tolist()
+                labels_list = list(self.labels_)
                 ips = []
                 for n in self.unique_labels_:
                     cluster_ions = labels_list.count(n)
@@ -419,6 +452,9 @@ class GaussianMixtureBase(metaclass=ABCMeta):
                 # Recalculate centers
                 self.recalculate_centers_uncertainties(data_frame_object=data_frame_object)
 
+                if self.n_comps_found_ == 1:
+                    break
+
                 # Generate new figure
                 fig, save_string = self.show_results(data_frame_object=data_frame_object)
 
